/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.tapschema.*;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.Model;

/**
 * SchemaPage class extends the default BasePage class.
 * SchemaPage class has an AjaxTabbedPanel to show the SchemaTabs dinamically,
 * and has the AjaxSubmitLink to submit all the tapschema structure to the
 * tapschema database.
 * 
 * @author pietro
 */
public class SchemaPage extends BasePage{
    
    Form schemaForm;
    
    final List<ITab> schemataTabs;
    final AjaxTabbedPanel schemataTabbedPanel;
    
    AjaxSubmitLink updateSubmit;
    AjaxSubmitLink backSubmit;
    TextArea infoTextArea;
    
    public TapSchema tapschema;
    
    /**
     * SchemaPage constructor
     * 
     * @param target
     * @param form
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param tapschema tapschema structure
     */
    public SchemaPage(
            AjaxRequestTarget target,
            Form<?> form,
            final String hostname,
            final String port,
            final String username,
            final String password,
            final String tapschemaName,
            final TapSchema tapschema)
    {
        this.tapschema = tapschema;
        
        //utype label
        Label utypeLabel = new Label("titleLabel", new Model(tapschemaName));
        add(utypeLabel);
        
        //schema form
        schemaForm = new Form("schemaForm");
        add(schemaForm);
        
        //schemata AjaxTabbedPanel
        schemataTabs = new ArrayList<ITab>();
        schemataTabbedPanel = new AjaxTabbedPanel("schemataTabbedPanel", schemataTabs);
        schemataTabbedPanel.setOutputMarkupPlaceholderTag(true);
        schemaForm.add(schemataTabbedPanel);
        
        //update AjaxSubmitLink
        updateSubmit = new AjaxSubmitLink(
                "updateSubmit",
                schemaForm)
        {
            @Override
            protected void onError(AjaxRequestTarget target, Form<?> form)
            {
                
            }
            
            /**
             * onSubmit method of updateSubmit AjaxSubmitLink updates the tapschema database.
             * 
             * @param target
             * @param form
             */
            @Override
            protected void onSubmit(AjaxRequestTarget target, Form<?> form)
            {
                try
                {
                    UpdateTapSchema updateTapSchema = new UpdateTapSchema();
                    boolean alert = updateTapSchema.controlChecked(target, tapschema);
                    
                    if (alert == false)
                    {
                        /*
                        updateTapSchema.createTapSchemaDatabase(
                                hostname,
                                port,
                                username,
                                password,
                                tapschemaName);
                                */
                        // delete tables in tapschema database
                        updateTapSchema.resetTapSchemaDatabase(
                                hostname,
                                port,
                                username,
                                password,
                                tapschemaName);
                        
                        // inserts the metadata from the variables of
                        //tapschema structure to the tapschema database
                        updateTapSchema.updateTapSchemaDatabase(
                                hostname,
                                port,
                                username,
                                password,
                                tapschemaName,
                                tapschema);
                        infoTextArea.setModelObject(tapschemaName + " database updated");
                        target.add(infoTextArea);
                    }
                    
                }
                catch (Exception exception)
                {
                    infoTextArea.setModelObject((String)exception.getMessage());
                    target.add(infoTextArea);
                    exception.printStackTrace();
                }
            }
        };
        schemaForm.add(updateSubmit);
        
        //back AjaxSubmitLink
        backSubmit = new AjaxSubmitLink(
                "backSubmit",
                schemaForm)
        {
            @Override
            protected void onError(AjaxRequestTarget target, Form<?> form)
            {
                
            }
            
            /**
             * onSubmit method of backSubmit AjaxSubmitLink sends back to SelectSchemataPage.
             * 
             * @param target
             * @param form
             */
            @Override
            protected void onSubmit(AjaxRequestTarget target, Form<?> form)
            {
                try
                {
                    SelectSchemataPage selectSchemataPage = new SelectSchemataPage(
                            target,
                            form,
                            hostname,
                            port,
                            username,
                            password);
                    setResponsePage(selectSchemataPage);
                }
                catch (Exception exception)
                {
                    infoTextArea.setModelObject((String)exception.getMessage());
                    target.add(infoTextArea);
                    exception.printStackTrace();
                }
            }
        };
        schemaForm.add(backSubmit);
        
        //info textarea
        infoTextArea = new TextArea("infoTextArea", Model.of(""));
        infoTextArea.setOutputMarkupPlaceholderTag(true);
        infoTextArea.setEnabled(false);
        schemaForm.add(infoTextArea);
    }
    
    /**
     * populateSchemaTabs method creates and populates SchemaTabs dinamically.
     * 
     * @param target
     * @param form
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param tapschema tapschema structure
     */
    public void populateSchemaTabs(
            final AjaxRequestTarget target,
            final Form<?> form,
            final String hostname,
            final String port,
            final String username,
            final String password,
            final String tapschemaName,
            final TapSchema tapschema)
    {
        try
        {
            //create schemataTabs dynamically
            schemataTabs.removeAll(schemataTabs);
            for (Schema schema : tapschema.schemasMap.values())
            {                
                String panelId = "panel";
                
                SchemaTab schemaTab = new SchemaTab(
                        target,
                        form,
                        panelId,
                        hostname,
                        port,
                        username,
                        password,
                        tapschemaName,
                        tapschema,
                        schema);
                
                schemataTabs.add(schemaTab);
            }
            target.add(schemataTabbedPanel);                    
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
        }
    }
    
    /**
     * populateTableTabs method creates and populates TableTabs dinamically.
     * 
     * @param target
     * @param form
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param schema current schema structure
     * @param schemaPanel 
     */
    public void populateTableTabs(
            AjaxRequestTarget target,
            Form<?> form,
            final String hostname,
            final String port,
            final String username,
            final String password,
            final String tapschemaName,
            final Schema schema,
            SchemaPanel schemaPanel)
    {
        try
        {
            //populates tablesTabs
            List<ITab> tablesTabs = schemaPanel.tablesTabs;
            tablesTabs.removeAll(tablesTabs);
            for (Table table : schema.tablesMap.values())
            {
                final String table_name = table.table_name;

                //populates rowsList
                List<Row> rowsList = new ArrayList<Row>();
                rowsList.removeAll(rowsList);
                for (Column column : table.columnsMap.values())
                {
                    String column_name = column.column_name;
                    
                    String primary_key = "";
                    //if (column.indexed == 1)
                    if (column.principal == 1)
                    {
                        primary_key = "pk";
                    }
                                        
                    String foreign_key = "";
                    for (Key key : schema.keysMap.values())
                    {
                        if (key.from_table.equals(table_name))
                        {
                            for (KeyColumn key_column : schema.key_columnsMap.values())
                            {
                                if (key_column.key_id.equals(key.key_id))
                                {
                                    if (key_column.from_column.equals(column_name))
                                    {
                                        foreign_key = "fk";
                                    }
                                }
                            }
                        }
                    }
                    
                    rowsList.add(new Row(
                            column.checked,
                            primary_key,
                            foreign_key,
                            table_name,
                            column_name,
                            column.utype,
                            column.ucd,
                            column.unit,
                            column.description,
                            column.datatype,
                            column.size,
                            column.principal,
                            column.indexed,
                            column.std));
                }

                //creates tablesTabs dynamically
                TableTab tableTab = new TableTab(
                        rowsList,
                        hostname,
                        port,
                        username,
                        password,
                        tapschemaName,
                        schema,
                        table);

                tablesTabs.add(tableTab);
            }//end for
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
        }
    }
    
}
