/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.tapschema.Schema;
import it.inaf.oats.ia2.tap.utils.tapschema.TapSchema;
import java.util.ArrayList;
import java.util.List;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;

/**
 * SchemaPanel class extends Panel class, to have a panel for every schema.
 * SchemaPanel class has an AjaxTabbedPanel to show the TableTabs dinamically.
 * 
 * @author pietro
 */
public class SchemaPanel extends Panel {
    
    final UtypeDescription utypeDescription;
    
    public final AjaxTextField utypeText;
    public final AjaxTextField descriptionText;
    public final AjaxCheckBox selectSchemaCheckBox;
    List<ITab> tablesTabs;
    public AjaxTabbedPanel tablesTabbedPanel;
    
    public String schema_name;
    
    /**
     * SchemaPanel constructor
     * 
     * @param id
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param tapschema tapschema structure
     * @param schema current schema structure
     */
    public SchemaPanel(
            String id,
            final String hostname,
            final String port,
            final String username,
            final String password,
            final String tapschemaName,
            final TapSchema tapschema,
            Schema schema)
    {
        super(id);
        
        this.schema_name = schema.schema_name;
        
        utypeDescription = new UtypeDescription(schema.utype, schema.description);
        
        //update schema form
        Form updateSchemaForm = new Form("updateSchemaForm");
        add(updateSchemaForm);
        
        //utype label
        Label utypeLabel = new Label("utypeLabel", new Model(schema_name + " utype"));
        updateSchemaForm.add(utypeLabel);
        
        //utype textfield
        utypeText = new AjaxTextField(
                "utypeText",
                new PropertyModel(utypeDescription, "utype"));
        utypeText.setIdString(
                schema.schema_name + "-" + 
                "utype");
        utypeText.setOutputMarkupPlaceholderTag(true);
        utypeText.add(new AttributeAppender("style", "width: 400px"));
        updateSchemaForm.add(utypeText);
        
        //description label
        Label descriptionLabel = new Label("descriptionLabel", new Model(schema_name + " description"));
        updateSchemaForm.add(descriptionLabel);
        
        //description textfield
        descriptionText = new AjaxTextField(
                "descriptionText",
                new PropertyModel(utypeDescription, "description"));
        descriptionText.setIdString(
                schema.schema_name + "-" + 
                "description");
        descriptionText.setOutputMarkupPlaceholderTag(true);
        descriptionText.add(new AttributeAppender("style", "width: 400px"));
        updateSchemaForm.add(descriptionText);
        
        //select schema label
        Label selectSchemaLabel = new Label("selectSchemaLabel", new Model("Select " + schema_name));
        updateSchemaForm.add(selectSchemaLabel);
        
        //select schema checkbox
        selectSchemaCheckBox = new AjaxCheckBox("selectSchemaCheckBox", new Model(schema.checked));
        selectSchemaCheckBox.setIdString(
                schema.schema_name + "-" +
                "checked");
        selectSchemaCheckBox.add(new AjaxFormComponentUpdatingBehavior("onchange")
        {
            /**
             * onUpdate method of selectSchemaCheckBox sets the  variable "checked"
             * in the correct schema of tapschema structure
             * 
             * @param target
             */
            @Override
            protected void onUpdate(AjaxRequestTarget target)
            {
                SchemaPage schemaPage = (SchemaPage)getPage();
                String[] idSplit = selectSchemaCheckBox.getIdString().split("-");
                if (idSplit[1].equals("checked"))
                {
                    schemaPage.tapschema.schemasMap.get(idSplit[0])
                            .checked
                            = (boolean)selectSchemaCheckBox.getModelObject();
                }
            }
        });
        selectSchemaCheckBox.setOutputMarkupPlaceholderTag(true);
        updateSchemaForm.add(selectSchemaCheckBox);
        
        //tables AjaxTabbedPanel
        tablesTabs = new ArrayList<ITab>();
        tablesTabbedPanel = new AjaxTabbedPanel("tablesTabbedPanel", tablesTabs);
        tablesTabbedPanel.setOutputMarkupPlaceholderTag(true);
        add(tablesTabbedPanel);
    }
    
}
