/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.tapschema.Column;
import it.inaf.oats.ia2.tap.utils.tapschema.Schema;
import it.inaf.oats.ia2.tap.utils.tapschema.Table;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.DataGridView;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.PropertyPopulator;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;

/**
 * TablePanel class extends Panel class, to have a panel for every table and many
 * tables for every schema.
 * TablePanel class has a DataGridView to show the metadata of "columns" table.
 * 
 * @author pietro
 */
public class TablePanel extends Panel {
    
    final AjaxTextField utypeText2;
    final AjaxTextField descriptionText2;
    final AjaxCheckBox selectTableCheckBox;
    public final DataGridView<Row> rowDataGridView;
    final AjaxCheckBox selectAllColumnsCheckBox;
    
    public String table_name;
    
    /**
     * TablePanel constructor
     * 
     * @param id
     * @param rowsList list of rows to pass to the ListDataProvider of the DataGridView
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param schema current schema structure
     * @param table current table structure
     */
    public TablePanel(
            String id,
            List<Row> rowsList,
            final String hostname,
            final String port,
            final String username,
            final String password,
            final String tapschemaName,
            final Schema schema,
            final Table table)
    {
        super(id);
        
        this.table_name = table.table_name;
        
        //populate columnsList
        final List<Column> columnsList = new ArrayList<Column>();
        columnsList.removeAll(columnsList);
        for (Column column : table.columnsMap.values())
        {
            columnsList.add(column);
        }
        
        // lists needed to memorize the item (row) indexes
        final List<Integer> selectItemIndexList = new ArrayList<Integer>();
        final List<Integer> primaryItemIndexList = new ArrayList<Integer>();
        final List<Integer> foreignItemIndexList = new ArrayList<Integer>();
        final List<Integer> tableItemIndexList = new ArrayList<Integer>();
        final List<Integer> columnItemIndexList = new ArrayList<Integer>();
        final List<Integer> utypeItemIndexList = new ArrayList<Integer>();
        final List<Integer> ucdItemIndexList = new ArrayList<Integer>();
        final List<Integer> unitItemIndexList = new ArrayList<Integer>();
        final List<Integer> descriptionItemIndexList = new ArrayList<Integer>();
        final List<Integer> datatypeItemIndexList = new ArrayList<Integer>();
        final List<Integer> sizeItemIndexList = new ArrayList<Integer>();
        final List<Integer> principalItemIndexList = new ArrayList<Integer>();
        final List<Integer> indexedItemIndexList = new ArrayList<Integer>();
        final List<Integer> stdItemIndexList = new ArrayList<Integer>();
        
        UtypeDescription utypeDescription = new UtypeDescription(table.utype, table.description);
        
        //update table form
        Form updateTableForm = new Form("updateTableForm");
        add(updateTableForm);
        
        //utype label
        Label utypeLabel2 = new Label("utypeLabel2", new Model(table_name + " utype"));
        updateTableForm.add(utypeLabel2);
        
        //utype textfield
        utypeText2 = new AjaxTextField(
                "utypeText2",
                new PropertyModel(utypeDescription, "utype"));
        utypeText2.setIdString(
                schema.schema_name + "-" + 
                table.table_name + "-" +
                "utype");
        utypeText2.setOutputMarkupPlaceholderTag(true);
        utypeText2.add(new AttributeAppender("style", "width: 400px"));
        updateTableForm.add(utypeText2);
        
        //description label
        Label descriptionLabel2 = new Label("descriptionLabel2", new Model(table_name + " description"));
        updateTableForm.add(descriptionLabel2);
        
        //description textfield
        descriptionText2 = new AjaxTextField(
                "descriptionText2",
                new PropertyModel(utypeDescription, "description"));
        descriptionText2.setIdString(
                schema.schema_name + "-" + 
                table.table_name + "-" +
                "description");
        descriptionText2.setOutputMarkupPlaceholderTag(true);
        descriptionText2.add(new AttributeAppender("style", "width: 400px"));
        updateTableForm.add(descriptionText2);
        
        //select schema label
        Label selectTableLabel = new Label("selectTableLabel", new Model("Select " + table_name));
        updateTableForm.add(selectTableLabel);
        
        //select table checkbox
        selectTableCheckBox = new AjaxCheckBox("selectTableCheckBox", new Model(table.checked));
        selectTableCheckBox.setIdString(
                schema.schema_name + "-" + 
                table.table_name + "-" +
                "checked");
        selectTableCheckBox.add(new AjaxFormComponentUpdatingBehavior("onchange")
        {
            /**
             * onUpdate method of AjaxFormComponentUpdatingBehavior on the onchange event
             * sets variable "checked" in the correct schema of tapschema structure
             * and sets variable "checked" in the correct table of tapschema structure.
             * 
             * @param target
             */
            @Override
            protected void onUpdate(AjaxRequestTarget target)
            {
                //sets variable "checked" in the correct schema of tapschema structure
                if (selectTableCheckBox.getModelObject() == true)
                {
                    SchemaPanel schemaPanel = (SchemaPanel)getParent().getParent();
                    schemaPanel.selectSchemaCheckBox.setModelObject(true);
                    
                    SchemaPage schemaPage = (SchemaPage)getPage();
                    String[] idSplit = schemaPanel.selectSchemaCheckBox.getIdString().split("-");
                    if (idSplit[1].equals("checked"))
                    {
                        schemaPage.tapschema.schemasMap.get(idSplit[0])
                                .checked
                                = (boolean)schemaPanel.selectSchemaCheckBox.getModelObject();
                    }
                    
                    target.add(schemaPanel.selectSchemaCheckBox);
                }
                //end setting variable "checked"
                    
                //sets variable "checked" in the correct table of tapschema structure
                SchemaPage schemaPage = (SchemaPage)getPage();
                String[] idSplit = selectTableCheckBox.getIdString().split("-");
                if (idSplit[2].equals("checked"))
                {
                    schemaPage.tapschema.schemasMap.get(idSplit[0])
                            .tablesMap.get(idSplit[1])
                            .checked
                            = (boolean)selectTableCheckBox.getModelObject();
                }
                //end setting variable "checked"
            }
        });
        selectTableCheckBox.setOutputMarkupPlaceholderTag(true);
        updateTableForm.add(selectTableCheckBox);
        
        //headers RepeatingView in "rows" DataGridView
        RepeatingView headersRepeatingView = new RepeatingView("headersRepeatingView");
        headersRepeatingView.setOutputMarkupPlaceholderTag(true);
        updateTableForm.add(headersRepeatingView);
        headersRepeatingView.add(new Label("select", "select"));
        headersRepeatingView.add(new Label("pk", "pk"));
        headersRepeatingView.add(new Label("fk", "fk"));
        headersRepeatingView.add(new Label("table_name", "table_name"));
        headersRepeatingView.add(new Label("column_name", "column_name"));
        headersRepeatingView.add(new Label("utype", "utype"));
        headersRepeatingView.add(new Label("ucd", "ucd"));
        headersRepeatingView.add(new Label("unit", "unit"));
        headersRepeatingView.add(new Label("description", "description"));
        headersRepeatingView.add(new Label("datatype", "datatype"));
        headersRepeatingView.add(new Label("size", "size"));
        headersRepeatingView.add(new Label("principal", "principal"));
        headersRepeatingView.add(new Label("indexed", "indexed"));
        headersRepeatingView.add(new Label("std", "std"));
        
        //try with DataGridView
        final List<ICellPopulator<Row>> columns = new ArrayList<ICellPopulator<Row>>();
        
        //select column
        PropertyPopulator<Row> selectPropertyPopulator = new PropertyPopulator<Row>("select")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new Model(""));
                cell.setVisible(false);
                cellItem.add(cell);
                
                final AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new PropertyModel(rowModel, "select"));
                
                //set CheckBox idString in the correct item (row) of "rows" DataGridView
                if (selectItemIndexList.size() >= columnsList.size())
                {
                    selectItemIndexList.removeAll(selectItemIndexList);
                }
                cellItem.setIndex(selectItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell2.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "checked");
                selectItemIndexList.add(selectItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell2.add(new AjaxFormComponentUpdatingBehavior("onchange")
                {
                    /**
                     * onUpdate method of AjaxFormComponentUpdatingBehavior on the
                     * onchange event sets the table checkbox and the schema checkbox,
                     * and sets the corresponding variables of the tapschema structure.
                     * 
                     * @param target
                     */
                    @Override
                    protected void onUpdate(AjaxRequestTarget target)
                    {
                        if (cell2.getModelObject() == true)
                        {
                            //sets selectTableCheckBox
                            selectTableCheckBox.setModelObject(true);
                            SchemaPage schemaPage = (SchemaPage)getPage();
                            String idString = selectTableCheckBox.getIdString();
                            String[] idSplit = selectTableCheckBox.getIdString().split("-");
                            if (idSplit[2].equals("checked"))
                            {
                                schemaPage.tapschema.schemasMap.get(idSplit[0])
                                        .tablesMap.get(idSplit[1])
                                        .checked
                                        = (boolean)selectTableCheckBox.getModelObject();
                            }
                            target.add(selectTableCheckBox);
                            //end setting selectTableCheckBox
                            
                            //sets selectSchemaCheckBox
                            SchemaPanel schemaPanel = (SchemaPanel)getParent().getParent();
                            schemaPanel.selectSchemaCheckBox.setModelObject(true);
                            
                            schemaPage = (SchemaPage)getPage();
                            idString = schemaPanel.selectSchemaCheckBox.getIdString();
                            idSplit = schemaPanel.selectSchemaCheckBox.getIdString().split("-");
                            if (idSplit[1].equals("checked"))
                            {
                                schemaPage.tapschema.schemasMap.get(idSplit[0])
                                        .checked
                                        = (boolean)schemaPanel.selectSchemaCheckBox.getModelObject();
                            }                                                        
                            target.add(schemaPanel.selectSchemaCheckBox);
                            //end setting selectSchemaCheckBox
                        }
                        
                        SchemaPage schemaPage = (SchemaPage)getPage();
                        String idString = cell2.getIdString();
                        String[] idSplit = cell2.getIdString().split("-");
                        if (idSplit[3].equals("checked"))
                        {
                            schemaPage.tapschema.schemasMap.get(idSplit[0])
                                    .tablesMap.get(idSplit[1])
                                    .columnsMap.get(idSplit[2])
                                    .checked
                                    = (boolean)cell2.getModelObject();
                        }
                    }
                });
                cell2.setOutputMarkupPlaceholderTag(true);
                cell2.setVisible(true);
                cellItem.add(cell2);
            }
        };
        columns.add(selectPropertyPopulator);
        
        //primary key column
        PropertyPopulator<Row> primaryPropertyPopulator = new PropertyPopulator<Row>("primary_key")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "primary_key"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (primaryItemIndexList.size() >= columnsList.size())
                {
                    primaryItemIndexList.removeAll(primaryItemIndexList);
                }
                cellItem.setIndex(primaryItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "pk");
                primaryItemIndexList.add(primaryItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 30px"));
                
                cell.setEnabled(false);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(primaryPropertyPopulator);
        
        //foreign key column
        PropertyPopulator<Row> foreignPropertyPopulator = new PropertyPopulator<Row>("foreign_key")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "foreign_key"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (foreignItemIndexList.size() >= columnsList.size())
                {
                    foreignItemIndexList.removeAll(foreignItemIndexList);
                }
                cellItem.setIndex(foreignItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "fk");
                foreignItemIndexList.add(foreignItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 30px"));
                
                cell.setEnabled(false);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(foreignPropertyPopulator);
                
        //table_name column
        PropertyPopulator<Row> tablePropertyPopulator = new PropertyPopulator<Row>("table_name")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "table_name"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (tableItemIndexList.size() >= columnsList.size())
                {
                    tableItemIndexList.removeAll(tableItemIndexList);
                }
                cellItem.setIndex(tableItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "table_name");
                tableItemIndexList.add(tableItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.setEnabled(false);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(tablePropertyPopulator);
        
        //column_name column
        PropertyPopulator<Row> columnPropertyPopulator = new PropertyPopulator<Row>("column_name")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "column_name"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (columnItemIndexList.size() >= columnsList.size())
                {
                    columnItemIndexList.removeAll(columnItemIndexList);
                }
                cellItem.setIndex(columnItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "column_name");
                columnItemIndexList.add(columnItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.setEnabled(false);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(columnPropertyPopulator);
        
        //utype column
        PropertyPopulator<Row> utypePropertyPopulator = new PropertyPopulator<Row>("utype")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "utype"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (utypeItemIndexList.size() >= columnsList.size())
                {
                    utypeItemIndexList.removeAll(utypeItemIndexList);
                }
                cellItem.setIndex(utypeItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "utype");
                utypeItemIndexList.add(utypeItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 100px"));
                cell.add(new AttributeAppender("showtooltip", new Model("true"), ""));
                cell.add(new AttributeAppender("title", new Model((String)cell.getModelObject()), ""));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(utypePropertyPopulator);
        
        //ucd column
        PropertyPopulator<Row> ucdPropertyPopulator = new PropertyPopulator<Row>("ucd")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "ucd"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (ucdItemIndexList.size() >= columnsList.size())
                {
                    ucdItemIndexList.removeAll(ucdItemIndexList);
                }
                cellItem.setIndex(ucdItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "ucd");
                ucdItemIndexList.add(ucdItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 100px"));
                cell.add(new AttributeAppender("showtooltip", new Model("true"), ""));
                cell.add(new AttributeAppender("title", new Model((String)cell.getModelObject()), ""));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(ucdPropertyPopulator);
        
        //unit column
        PropertyPopulator<Row> unitPropertyPopulator = new PropertyPopulator<Row>("unit")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "unit"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (unitItemIndexList.size() >= columnsList.size())
                {
                    unitItemIndexList.removeAll(unitItemIndexList);
                }
                cellItem.setIndex(unitItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "unit");
                unitItemIndexList.add(unitItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 100px"));
                cell.add(new AttributeAppender("showtooltip", new Model("true"), ""));
                cell.add(new AttributeAppender("title", new Model((String)cell.getModelObject()), ""));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(unitPropertyPopulator);
        
        //description column
        PropertyPopulator<Row> descriptionPropertyPopulator = new PropertyPopulator<Row>("description")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "description"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (descriptionItemIndexList.size() >= columnsList.size())
                {
                    descriptionItemIndexList.removeAll(descriptionItemIndexList);
                }
                cellItem.setIndex(descriptionItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "description");
                descriptionItemIndexList.add(descriptionItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 100px"));
                cell.add(new AttributeAppender("showtooltip", new Model("true"), ""));
                cell.add(new AttributeAppender("title", new Model((String)cell.getModelObject()), ""));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(descriptionPropertyPopulator);
        
        //datatype column
        PropertyPopulator<Row> datatypePropertyPopulator = new PropertyPopulator<Row>("datatype")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "datatype"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (datatypeItemIndexList.size() >= columnsList.size())
                {
                    datatypeItemIndexList.removeAll(datatypeItemIndexList);
                }
                cellItem.setIndex(datatypeItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "datatype");
                datatypeItemIndexList.add(datatypeItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 100px"));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(datatypePropertyPopulator);
        
        //size column
        PropertyPopulator<Row> sizePropertyPopulator = new PropertyPopulator<Row>("size")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "size"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (sizeItemIndexList.size() >= columnsList.size())
                {
                    sizeItemIndexList.removeAll(sizeItemIndexList);
                }
                cellItem.setIndex(sizeItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "size");
                sizeItemIndexList.add(sizeItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 50px"));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(sizePropertyPopulator);
        
        //principal column
        PropertyPopulator<Row> principalPropertyPopulator = new PropertyPopulator<Row>("principal")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "principal"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (principalItemIndexList.size() >= columnsList.size())
                {
                    principalItemIndexList.removeAll(principalItemIndexList);
                }
                cellItem.setIndex(principalItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "principal");
                principalItemIndexList.add(principalItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 50px"));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(principalPropertyPopulator);
        
        //indexed column
        PropertyPopulator<Row> indexedPropertyPopulator = new PropertyPopulator<Row>("indexed")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "indexed"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (indexedItemIndexList.size() >= columnsList.size())
                {
                    indexedItemIndexList.removeAll(indexedItemIndexList);
                }
                cellItem.setIndex(indexedItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "indexed");
                indexedItemIndexList.add(indexedItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 50px"));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(indexedPropertyPopulator);
        
        //std column
        PropertyPopulator<Row> stdPropertyPopulator = new PropertyPopulator<Row>("std")
        {
            /**
             * populateItem method of the PropertyPopulator variable sets the data
             * and the idString variable of the cell components in the current item (row)
             * in the "rows" DataGridView.
             * 
             * @param cellItem cell item of the current item (row) in the "rows" DataGridView
             * @param componentId
             * @param rowModel model of the the current item (row) in the "rows" DataGridView
             */
            @Override
            public void populateItem(Item<ICellPopulator<Row>> cellItem, String componentId, IModel<Row> rowModel)
            {
                AjaxTextField cell = new AjaxTextField("cell", new PropertyModel(rowModel, "std"));
                
                //set TextField idString in the correct item (row) of "rows" DataGridView
                if (stdItemIndexList.size() >= columnsList.size())
                {
                    stdItemIndexList.removeAll(stdItemIndexList);
                }
                cellItem.setIndex(stdItemIndexList.size());
                Column column = null;
                for (int i = 0; i < columnsList.size(); i++)
                {
                    if (i == cellItem.getIndex())
                    {
                        column = columnsList.get(i);
                        break;
                    }
                }
                cell.setIdString(
                        schema.schema_name + "-" + 
                        table.table_name + "-" +
                        column.column_name + "-" +
                        "std");
                stdItemIndexList.add(stdItemIndexList.size() + 1);
                //end setting TextField idString
                
                cell.add(new AttributeAppender("style", "width: 50px"));
                
                cell.setOutputMarkupPlaceholderTag(true);
                cellItem.add(cell);
                
                AjaxCheckBox cell2 = new AjaxCheckBox("cell2", new Model(false));
                cell2.setVisible(false);
                cellItem.add(cell2);
            }
        };
        columns.add(stdPropertyPopulator);
        
        // "rows" DataGridView
        rowDataGridView = new DataGridView<Row>("rows", columns, new ListDataProvider(rowsList));
        updateTableForm.add(rowDataGridView);
        
        //select all table columns label
        Label selectAllColumnsLabel = new Label("selectAllColumnsLabel", new Model("Select all table columns"));
        updateTableForm.add(selectAllColumnsLabel);
        
        //select all table columns checkbox
        selectAllColumnsCheckBox = new AjaxCheckBox("selectAllColumnsCheckBox", new Model(false));
        selectAllColumnsCheckBox.add(new AjaxFormComponentUpdatingBehavior("onchange")
        {
            /**
             * onUpdate method of AjaxFormComponentUpdatingBehavior on the event onchange
             * sets the checkboxes of all the columns, of the current table and of the
             * current schema, and sets the corresponding "check" variables in the
             * tapschema structure.
             * 
             * @param target
             */
            @Override
            protected void onUpdate(AjaxRequestTarget target)
            {
                //sets column checkboxes
                Iterator iterator = rowDataGridView.getItems();
                List<Item> itemList = new ArrayList<Item>();
                while (iterator.hasNext())
                {
                    itemList.add((Item)iterator.next());
                }
                for (Item item : itemList)
                {
                    RepeatingView cells = (RepeatingView)item.get("cells");
                    AjaxCheckBox cell2 = (AjaxCheckBox)cells.get(0).get("cell2");
                    cell2.setModelObject(selectAllColumnsCheckBox.getModelObject());
                    target.add(cell2);
                }
                //end setting column checkboxes
                
                //sets variable "checked" in all the columns of tapschema structure4
                SchemaPage schemaPage = (SchemaPage)getPage();
                String[] idSplit = selectTableCheckBox.getIdString().split("-");
                for (Column column : schemaPage.tapschema.schemasMap.get(idSplit[0])
                        .tablesMap.get(idSplit[1]).columnsMap.values())
                {
                    column.checked = selectAllColumnsCheckBox.getModelObject();
                }
                //end setting variable "checked"
                
                if (selectAllColumnsCheckBox.getModelObject() == true)
                { 
                    //sets selectTableCheckBox
                    selectTableCheckBox.setModelObject(true);
                    schemaPage = (SchemaPage)getPage();
                    idSplit = selectTableCheckBox.getIdString().split("-");
                    if (idSplit[2].equals("checked"))
                    {
                        schemaPage.tapschema.schemasMap.get(idSplit[0])
                                .tablesMap.get(idSplit[1])
                                .checked
                                = (boolean)selectTableCheckBox.getModelObject();
                    }
                    target.add(selectTableCheckBox);
                    //end setting selectTableCheckBox

                    //sets selectSchemaCheckBox
                    SchemaPanel schemaPanel = (SchemaPanel)getParent().getParent();
                    schemaPanel.selectSchemaCheckBox.setModelObject(true);

                    schemaPage = (SchemaPage)getPage();
                    idSplit = schemaPanel.selectSchemaCheckBox.getIdString().split("-");
                    if (idSplit[1].equals("checked"))
                    {
                        schemaPage.tapschema.schemasMap.get(idSplit[0])
                                .checked
                                = (boolean)schemaPanel.selectSchemaCheckBox.getModelObject();
                    }                                                        
                    target.add(schemaPanel.selectSchemaCheckBox);
                    //end setting selectSchemaCheckBox
                }
            }
        });
        selectAllColumnsCheckBox.setOutputMarkupPlaceholderTag(true);
        updateTableForm.add(selectAllColumnsCheckBox);
    }
      
}
