/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.tapschema.*;
import java.sql.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.apache.wicket.ajax.AjaxRequestTarget;

/**
 * UpdateTapSchema class contains the methods to create, read and write
 * tapschema databases.
 * 
 * @author pietro
 */
public class UpdateTapSchema {
    
    public UpdateTapSchema()
    {
        
    }
    
    /**
     * createTapSchemaDatabase method creates a new tapschema database in MySQL on hostname.
     * It creates all the tapschema tables with their fields.
     * 
     * @param server
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @throws Exception 
     */
    public void createTapSchemaDatabase(
            String server,
            String port,
            String username,
            String password,
            String tapschemaName)
            throws Exception
    {
        String url = "jdbc:mysql://" + server + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );
            statement = connection.createStatement();

            //creates database
            String queryString = "CREATE DATABASE IF NOT EXISTS " + tapschemaName + ";";
            statement.executeUpdate(queryString);

            //creates schemas table
            queryString = "CREATE TABLE IF NOT EXISTS " + tapschemaName + ".`schemas` (" +
                    "schema_name   varchar(64), " +
                    "utype         varchar(512) NULL, " +
                    "description   varchar(512) NULL, " +
                    "schemaID      bigint, " +
                    "PRIMARY KEY (schema_name));";
            statement.executeUpdate(queryString);

            //creates tables table
            queryString = "CREATE TABLE IF NOT EXISTS " + tapschemaName + ".tables (" +
                    "schema_name   varchar(64), " +
                    "table_name    varchar(128), " +
                    "table_type    varchar(8), " +
                    "utype         varchar(512) NULL, " +
                    "description   varchar(512) NULL, " +
                    "tableID       bigint, " +
                    "PRIMARY KEY (table_name), " +
                    "FOREIGN KEY (schema_name) REFERENCES " + tapschemaName + ".`schemas` (schema_name));";
            statement.executeUpdate(queryString);

            //creates columns table
            queryString = "CREATE TABLE IF NOT EXISTS " + tapschemaName + ".columns (" +
                    "table_name    varchar(128), " +
                    "column_name   varchar(64), " +
                    "utype         varchar(512) NULL, " +
                    "ucd           varchar(64)  NULL, " +
                    "unit          varchar(64)  NULL, " +
                    "description   varchar(512) NULL, " +
                    "datatype      varchar(64)  NOT NULL, " +
                    "size          integer      NULL, " +
                    "principal     integer      NOT NULL, " +
                    "indexed       integer      NOT NULL, " +
                    "std           integer      NOT NULL, " +
                    "columnID      bigint, " +
                    "PRIMARY KEY (table_name, column_name), " +
                    "FOREIGN KEY (table_name) REFERENCES " + tapschemaName + ".tables (table_name));";
            statement.executeUpdate(queryString);

            //creates keys table
            queryString = "CREATE TABLE IF NOT EXISTS " + tapschemaName + ".keys (" +
                    "key_id        varchar(64), " +
                    "from_table    varchar(128) NOT NULL, " +
                    "target_table  varchar(128) NOT NULL, " +
                    "utype         varchar(512) NULL, " +
                    "description   varchar(512) NULL, " +
                    "keyID         bigint, " +
                    "PRIMARY KEY (key_id), " +
                    "FOREIGN KEY (from_table) REFERENCES " + tapschemaName + ".tables (table_name), " +
                    "FOREIGN KEY (target_table) REFERENCES " + tapschemaName + ".tables (table_name));";
            statement.executeUpdate(queryString);

            //creates key_columns table
            queryString = "CREATE TABLE IF NOT EXISTS " + tapschemaName + ".key_columns (" +
                    "key_id          varchar(64), " +
                    "from_column     varchar(64)   NOT NULL, " +
                    "target_column   varchar(64) NOT NULL, " +
                    "key_columnID    bigint, " +
                    "FOREIGN KEY (key_id) REFERENCES " + tapschemaName + ".keys (key_id));";
            statement.executeUpdate(queryString);
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            try
            {
                statement.close();
                connection.close();
            }
            catch (Exception exception)
            {
                exception.printStackTrace();
                throw new Exception(exception.getMessage());
            }
        }
    }
    
    /**
     * populateFromDatabases method obtains metadata from all selected databases
     * and updates the variables of the new tapschema structure.
     * 
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param selectedDatabases
     * @param create indicates if the tapschema database is new or already existing
     * @return
     * @throws Exception 
     */
    public TapSchema populateFromDatabases(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName,
            Collection<String> selectedDatabases,
            boolean create)
            throws Exception
    {
        // create new tapschema structure
        TapSchema tapschema = new TapSchema();
        
        List<Integer> keyIDList = new ArrayList<Integer>();
        
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        ResultSet tablesResultSet = null;
        ResultSet columnsResultSet = null;
        ResultSet foreignKeysResultSet = null;

        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );

            for (String databaseName : selectedDatabases)
            {
                // updates variables of current schema structure
                Schema schema = new Schema();
                schema.schema_name = databaseName;
                schema.checked = false;
                
                statement = connection.createStatement();
                String queryString = "SHOW TABLES FROM " + databaseName + ";";
                tablesResultSet = statement.executeQuery(queryString);
                while (tablesResultSet.next())
                {
                    // updates variables of current table structure
                    Table table = new Table();
                    table.schema_name = databaseName;
                    table.table_name = databaseName + "." + tablesResultSet.getString(1);
                    table.table_type = "table";

                    statement = connection.createStatement();
                    queryString = "SHOW COLUMNS FROM " + table.table_name + ";";
                    columnsResultSet = statement.executeQuery(queryString);

                    while (columnsResultSet.next())
                    {
                        // updates variables of current column structure
                        int principal = 0;
                        if (columnsResultSet.getString("Key").equals("PRI"))
                        {
                            principal = 1; //this column is primary key
                        }
                        
                        int indexed = 0;
                        if (columnsResultSet.getString("Key").equals("PRI") ||
                                columnsResultSet.getString("Key").equals("UNI") ||
                                columnsResultSet.getString("Key").equals("MUL"))
                        {
                            indexed = 1;
                        }

                        int size;
                        String datatype;
                        if(columnsResultSet.getString(2).startsWith("int")
                                || columnsResultSet.getString(2).startsWith("INT"))
                        {
                            size = 0;
                            datatype = "adql:INTEGER";
                        }
                        else if(columnsResultSet.getString(2).startsWith("smallint")
                                || columnsResultSet.getString(2).startsWith("SMALLINT"))
                        {
                            size = 0;
                            datatype = "adql:SMALLINT";
                        }
                        else if(columnsResultSet.getString(2).startsWith("bigint")
                                || columnsResultSet.getString(2).startsWith("BIGINT"))
                        {
                            size = 0;
                            datatype = "adql:BIGINT";
                        }
                        else if(columnsResultSet.getString(2).startsWith("float")
                                || columnsResultSet.getString(2).startsWith("FLOAT"))
                        {
                            size = 0;
                            datatype = "adql:REAL";
                        }
                        //else if(columnsResultSet.getString(2).contains("char")
                        //        || columnsResultSet.getString(2).contains("CHAR"))
                        else if(columnsResultSet.getString(2).startsWith("char")
                                || columnsResultSet.getString(2).startsWith("CHAR"))
                        {
                            int beginIndex = columnsResultSet.getString(2).indexOf('(');
                            int endIndex = columnsResultSet.getString(2).indexOf(')');
                            size = Integer.parseInt(columnsResultSet.getString(2)
                                    .substring(beginIndex + 1, endIndex));
                            datatype = "adql:CHAR";
                        }
                        else if(columnsResultSet.getString(2).startsWith("varchar")
                                || columnsResultSet.getString(2).startsWith("VARCHAR"))
                        {
                            int beginIndex = columnsResultSet.getString(2).indexOf('(');
                            int endIndex = columnsResultSet.getString(2).indexOf(')');
                            size = Integer.parseInt(columnsResultSet.getString(2)
                                    .substring(beginIndex + 1, endIndex));
                            datatype = "adql:VARCHAR";
                        }
                        else if(columnsResultSet.getString(2).contains("timestamp")
                                || columnsResultSet.getString(2).contains("TIMESTAMP"))
                        {
                            size = 0;
                            datatype = "adql:TIMESTAMP";
                        }
                        else
                        {
                            size = 0;
                            datatype = "adql:" + columnsResultSet.getString(2).toUpperCase();
                        }
                        
                        int std = 0;
                        if ((databaseName.equals(tapschemaName)) &&
                                !((columnsResultSet.getString(1).equals("schemaID")) ||
                                (columnsResultSet.getString(1).equals("tableID")) ||
                                (columnsResultSet.getString(1).equals("columnID")) ||
                                (columnsResultSet.getString(1).equals("keyID")) ||
                                (columnsResultSet.getString(1).equals("key_columnID"))))
                        {
                            std = 1;
                        }

                        Column column = new Column();
                        column.table_name = table.table_name;
                        column.column_name = columnsResultSet.getString(1);
                        column.datatype = datatype;
                        column.size = size;
                        column.principal = principal;
                        column.indexed = indexed;
                        column.std = std;
                        
                        table.columnsMap.put(column.column_name, column);
                    } //end while columnsResultSet

                    schema.tablesMap.put(table.table_name, table);
                } //end while tablesResultSet

                statement = connection.createStatement();
                queryString = "SELECT k.TABLE_NAME, k.COLUMN_NAME, "
                        + "k.REFERENCED_TABLE_NAME, k.REFERENCED_COLUMN_NAME "
                        + "FROM information_schema.TABLE_CONSTRAINTS i "
                        + "LEFT JOIN information_schema.KEY_COLUMN_USAGE k "
                        + "ON i.CONSTRAINT_NAME = k.CONSTRAINT_NAME "
                        + "WHERE i.CONSTRAINT_TYPE = 'FOREIGN KEY' "
                        + "AND i.TABLE_SCHEMA = '" + databaseName + "' "
                        + "AND k.TABLE_SCHEMA = '" + databaseName + "';";
                foreignKeysResultSet = statement.executeQuery(queryString);

                while(foreignKeysResultSet.next())
                {
                    String from_table = foreignKeysResultSet.getString("k.TABLE_NAME");
                    String target_table = foreignKeysResultSet.getString("k.REFERENCED_TABLE_NAME");
                    String from_column = foreignKeysResultSet.getString("k.COLUMN_NAME"); 
                    String target_column = foreignKeysResultSet.getString("k.REFERENCED_COLUMN_NAME");

                    //calculate the max key_ID
                    boolean k = false;
                    if (create == false) //tapschema already exists
                    {
                        statement = connection.createStatement();
                        queryString = "SELECT key_ID FROM " + tapschemaName + ".keys;";
                        ResultSet resultSet = statement.executeQuery(queryString);
                        while(resultSet.next())
                        {
                            String key_ID = resultSet.getString(1);
                            if (key_ID.startsWith("k"))
                            {
                                k = true;
                                key_ID = key_ID.replaceAll("k", "");
                            }
                            else
                            {
                                k = false;
                            }
                            //keyIDList.add(Integer.parseInt(resultSet.getString(1)));
                            keyIDList.add(Integer.parseInt(key_ID));
                        }
                    }

                    Integer max = 0;
                    if(keyIDList.size() > 0)
                    {
                        max = keyIDList.get(0);
                    }

                    if(max != null && max != 0)
                    {
                        for (int m = 1; m < keyIDList.size(); m++)
                        {
                            if(keyIDList.get(m) > max)
                            {
                                max = keyIDList.get(m);
                            }
                        }
                    }

                    //obtain the new key_ID
                    if(max != null && max != 0)
                    {
                        max = max + 1;
                    }
                    else
                    {
                        max = 1;
                    }

                    keyIDList.add(max);

                    // updates variables of current key and key_column structure
                    Key key = new Key();
                    if (k == true)
                    {
                        key.key_id = "k" + max.toString();
                    }
                    else
                    {
                        key.key_id = max.toString();
                    }
                    key.from_table = databaseName + "." + from_table;
                    key.target_table = databaseName + "." + target_table;
                    schema.keysMap.put(key.key_id, key);

                    KeyColumn key_column = new KeyColumn();
                    if (k == true)
                    {
                        key_column.key_id = "k" + max.toString();
                    }
                    else
                    {
                        key_column.key_id = max.toString();
                    }
                    key_column.from_column = from_column;
                    key_column.target_column = target_column;
                    schema.key_columnsMap.put(key_column.key_id, key_column);
                } //end while foreignKeysResultSet
                
                tapschema.schemasMap.put(schema.schema_name, schema);
            } //end for selectedDatabases
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            tablesResultSet.close();
            columnsResultSet.close();
            foreignKeysResultSet.close();
            statement.close();
            connection.close();
        }
                
        return tapschema;
    } //end populateFromDatabases
    
    /**
     * populateFromTapSchema method obtains metadata from the tapschema database
     * and updates the variables of the existing tapschema structure.
     * 
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param selectedDatabases
     * @param create indicates if the tapschema database is new or already existing
     * @param tapschema tapschema structure
     * @return
     * @throws Exception 
     */
    public TapSchema populateFromTapSchema(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName,
            Collection<String> selectedDatabases,
            boolean create,
            TapSchema tapschema)
            throws Exception
    {
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        ResultSet schemasResultSet = null;
        ResultSet tablesResultSet = null;
        ResultSet columnsResultSet = null;
        ResultSet keysResultSet = null;
        ResultSet foreignKeysResultSet = null;

        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );

            for (Schema schema : tapschema.schemasMap.values())
            {
                //search if schema exists in tapschema
                statement = connection.createStatement();
                String queryString = "SELECT * FROM " + tapschemaName + ".`schemas`;";
                schemasResultSet = statement.executeQuery(queryString);
                while (schemasResultSet.next())
                {
                    if (schema.schema_name.equals(schemasResultSet.getString("schema_name")))
                    {
                        // existing schema in tapschema: loading metadata from tapschema
                        // updates variables of current schema structure
                        schema.utype = schemasResultSet.getString("utype");
                        schema.description = schemasResultSet.getString("description");
                        
                        schema.checked = true;

                        for (Table table : schema.tablesMap.values())
                        {
                            //search if table exists in schema
                            statement = connection.createStatement();
                            queryString = "SELECT * FROM " + tapschemaName + ".tables " + 
                                    "WHERE schema_name='" + schema.schema_name + "';";
                            tablesResultSet = statement.executeQuery(queryString);
                            while (tablesResultSet.next())
                            {
                                if (table.table_name.equals(tablesResultSet.getString("table_name")))
                                {
                                    // updates variables of current table structure
                                    table.table_name = tablesResultSet.getString("table_name");
                                    table.utype = tablesResultSet.getString("utype");
                                    table.description = tablesResultSet.getString("description");
                                    
                                    table.checked = true;

                                    for (Column column : table.columnsMap.values())
                                    {
                                        //search if column exists in table
                                        statement = connection.createStatement();
                                        queryString = "SELECT * FROM " + tapschemaName + ".columns " + 
                                                "WHERE table_name='" + table.table_name + "';";
                                        columnsResultSet = statement.executeQuery(queryString);

                                        while (columnsResultSet.next())
                                        {
                                            if (column.column_name.equals(columnsResultSet.getString("column_name")))
                                            {
                                                // updates variables of current column structure
                                                column.table_name = columnsResultSet.getString("table_name");
                                                column.column_name = columnsResultSet.getString("column_name");
                                                column.utype = columnsResultSet.getString("utype");
                                                column.ucd = columnsResultSet.getString("ucd");
                                                column.unit = columnsResultSet.getString("unit");
                                                column.description = columnsResultSet.getString("description");
                                                column.datatype = columnsResultSet.getString("datatype");
                                                column.size = columnsResultSet.getInt("size");
                                                column.principal = columnsResultSet.getInt("principal");
                                                column.indexed = columnsResultSet.getInt("indexed");
                                                column.std = columnsResultSet.getInt("std");
                                                
                                                column.checked = true;

                                                //search if foreign key
                                                statement = connection.createStatement();
                                                queryString = "SELECT * FROM " + tapschemaName + ".keys " +
                                                        "INNER JOIN " + tapschemaName + ".key_columns " +
                                                        "USING (key_id) " +
                                                        "WHERE " + tapschemaName + ".keys.from_table='" + column.table_name + "' " +
                                                        "AND " + tapschemaName + ".key_columns.from_column='" + column.column_name + "';";
                                                keysResultSet = statement.executeQuery(queryString);
                                                
                                                statement = connection.createStatement();
                                                queryString = "SELECT * FROM " + tapschemaName + ".keys " +
                                                        "INNER JOIN " + tapschemaName + ".key_columns " +
                                                        "USING (key_id) " +
                                                        "WHERE " + tapschemaName + ".keys.target_table='" + column.table_name + "' " +
                                                        "AND " + tapschemaName + ".key_columns.target_column='" + column.column_name + "';";
                                                foreignKeysResultSet = statement.executeQuery(queryString);
                                                while (keysResultSet.next())
                                                {
                                                    while (foreignKeysResultSet.next())
                                                    {
                                                        for (Key key : schema.keysMap.values())
                                                        {
                                                            for (KeyColumn key_column : schema.key_columnsMap.values())
                                                            {
                                                                if (key.from_table.equals(keysResultSet.getString("from_table"))
                                                                        && key_column.from_column.equals(foreignKeysResultSet.getString("from_column"))
                                                                        && keysResultSet.getString("key_id").equals(foreignKeysResultSet.getString("key_id")))
                                                                {
                                                                    // updates variables of current key structure
                                                                    key.key_id = keysResultSet.getString("key_id");
                                                                    key.utype = keysResultSet.getString("utype");
                                                                    key.description = keysResultSet.getString("description");
                                                                    
                                                                    // updates variables of current key_column structure
                                                                    key_column.key_id = foreignKeysResultSet.getString("key_id");
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            } //end if existing column in table
                                        } //end while columnsResultSet
                                    } //end for column in columnsList
                                } //end if existing table in schema
                            } //end while tablesResultSet         
                        } //end for table in tablesList
                    } //end if existing schema in tapschema
                } //end while schemasResultSet
            } //end for schema in schemasList
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            schemasResultSet.close();
            tablesResultSet.close();
            columnsResultSet.close();
            keysResultSet.close();
            foreignKeysResultSet.close();
            statement.close();
            connection.close();
        }
        
        return tapschema;
    } //end populateFromTapSchema
    
    /**
     * resetTapSchemaDatabase method resets the tapschema database by deleting
     * all its tables.
     * 
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @throws Exception 
     */
    public void resetTapSchemaDatabase(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName)
            throws Exception
    {
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        
        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );
            
            // deletes key_columns table
            statement = connection.createStatement();
            String queryString = "DELETE FROM " + tapschemaName + ".key_columns;";
            statement.executeUpdate(queryString);
            
            // deletes keys table
            statement = connection.createStatement();
            queryString = "DELETE FROM " + tapschemaName + ".keys;";
            statement.executeUpdate(queryString);
            
            // deletes columns table
            statement = connection.createStatement();
            queryString = "DELETE FROM " + tapschemaName + ".columns;";
            statement.executeUpdate(queryString);
            
            // deletes tables table
            statement = connection.createStatement();
            queryString = "DELETE FROM " + tapschemaName + ".tables;";
            statement.executeUpdate(queryString);
            
            // deletes schemas table
            statement = connection.createStatement();
            queryString = "DELETE FROM " + tapschemaName + ".`schemas`;";
            statement.executeUpdate(queryString);
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            statement.close();
            connection.close();
        }
    }
    
    /**
     * updateTapSchemaDatabase method inserts the metadata from the variables of
     * tapschema structure to the tapschema database.
     * 
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @param tapschema tapschema structure
     * @throws Exception 
     */
    public void updateTapSchemaDatabase(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName,
            TapSchema tapschema)
            throws Exception
    {
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        
        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );
            
            for (Schema schema : tapschema.schemasMap.values())
            {
                if (schema.checked == true)
                {
                    if (schema.utype == null)
                    {
                        schema.utype = "";
                    }
                    if (schema.description == null)
                    {
                        schema.description = "";
                    }
                    // inserts metadata in schemas table
                    statement = connection.createStatement();
                    String queryString = "INSERT INTO " + tapschemaName + ".`schemas` "
                            + "(schema_name, utype, description, schemaID) VALUES('"
                            + schema.schema_name + "', '"
                            + schema.utype + "', '"
                            + schema.description + "', "
                            + "null);";
                    statement.executeUpdate(queryString);
                    
                    for (Table table : schema.tablesMap.values())
                    {
                        if (table.checked == true)
                        {
                            if (table.utype == null)
                            {
                                table.utype = "";
                            }
                            if (table.description == null)
                            {
                                table.description = "";
                            }
                            // inserts metadata in tables table
                            statement = connection.createStatement();
                            queryString = "INSERT INTO " + tapschemaName + ".tables " +
                                    "(schema_name, table_name, table_type, utype, description, tableID) "
                                    + "VALUES('"
                                    + table.schema_name + "', '"
                                    + table.table_name + "', "
                                    + "'table', '"
                                    + table.utype + "', '"
                                    + table.description + "', "
                                    + "null);";
                            statement.executeUpdate(queryString);
                            
                            for (Column column : table.columnsMap.values())
                            {
                                if (column.checked == true)
                                {
                                    if (column.utype == null)
                                    {
                                        column.utype = "";
                                    }
                                    if (column.ucd == null)
                                    {
                                        column.ucd = "";
                                    }
                                    if (column.unit == null)
                                    {
                                        column.unit = "";
                                    }
                                    if (column.description == null)
                                    {
                                        column.description = "";
                                    }
                                    if (column.datatype == null)
                                    {
                                        column.datatype = "";
                                    }
                                    // inserts metadata in columns table
                                    statement = connection.createStatement();
                                    queryString = "INSERT INTO " + tapschemaName + ".columns "
                                            + "(table_name, column_name, utype, ucd, unit, description, "
                                            + "datatype, size, principal, indexed, std, columnID) "
                                            + "VALUES('"
                                            + column.table_name + "', '"
                                            + column.column_name + "', '"
                                            + column.utype + "', '"
                                            + column.ucd + "', '"
                                            + column.unit + "', '"
                                            + column.description + "', '"
                                            + column.datatype + "', "
                                            + column.size + ", "
                                            + column.principal + ", "
                                            //+ 0 + ", "
                                            + column.indexed + ", "
                                            + column.std + ", "
                                            + "null);";
                                    statement.executeUpdate(queryString);
                                } //end if column.checked
                            } //end for columnsMap
                        } //end if table.checked
                    } //end for tablesMap
                    
                    for (Key key : schema.keysMap.values())
                    {
                        for (KeyColumn key_column : schema.key_columnsMap.values())
                        {
                            for (Table table : schema.tablesMap.values())
                            {
                                for (Column column : table.columnsMap.values())
                                {
                                    if (column.column_name.equals(key_column.from_column)
                                            && column.table_name.equals(key.from_table)
                                            && key.key_id.equals(key_column.key_id)
                                            && table.checked == true
                                            && column.checked == true)
                                    {
                                        if (key.utype == null)
                                        {
                                            key.utype = "";
                                        }
                                        if (key.description == null)
                                        {
                                            key.description = "";
                                        }
                                        // inserts metadata in keys table
                                        statement = connection.createStatement();
                                        queryString = "INSERT INTO " + tapschemaName + ".keys " +
                                                "(key_id, from_table, target_table, utype, description, keyID) "
                                                + "VALUES('"
                                                + key.key_id + "', '"
                                                + key.from_table + "', '"
                                                + key.target_table + "', '"
                                                + key.utype + "', '"
                                                + key.description + "', "
                                                + "null);";
                                        statement.executeUpdate(queryString);
                                        
                                        // inserts metadata in key_columns table
                                        statement = connection.createStatement();
                                        queryString = "INSERT INTO " + tapschemaName + ".key_columns " +
                                                "(key_id, from_column, target_column, key_columnID) "
                                                + "VALUES('"
                                                + key_column.key_id + "', '"
                                                + key_column.from_column + "', '"
                                                + key_column.target_column + "', "
                                                + "null);";
                                        statement.executeUpdate(queryString);
                                    } //end if
                                } //end for columnsMap
                            } //end for tablesMap
                        } //end for key_columnsMap
                    } //end for keysMap
                } //end if schema.checked
            } //end for schemasMap
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            statement.close();
            connection.close();
        }
    }
    
    /**
     * controlChecked method shows an alert message if a table checkbox is checked
     * and the schema checkbox not, or if a column checkbox is checked and the table
     * checkbox not, or if a column checkbox is checked and the schema checkbox not.
     * 
     * @param target
     * @param tapschema tapschema structure
     * @return 
     */
    public boolean controlChecked(AjaxRequestTarget target, TapSchema tapschema)
    {
        boolean alert = false;
        String alertString;
        for (Schema schema : tapschema.schemasMap.values())
        {
            for (Table table : schema.tablesMap.values())
            {
                if (table.checked == true && schema.checked == false)
                {
                    alertString = "Can not update table " +
                            table.table_name +
                            " because schema " + schema.schema_name +
                            " is not checked.\n" +
                            " Check the schema if you want to update the schema," +
                            " or uncheck the table if you don't want to update the table.";
                    target.appendJavaScript(";alert('" + alertString + "');");
                    alert = true;
                    return alert;
                }
                
                for (Column column : table.columnsMap.values())
                {
                    if (column.checked == true && table.checked == false)
                    {
                        alertString = "Can not update column " +
                                column.table_name + "." + column.column_name +
                                " because table " + table.table_name +
                                " is not checked.\n" +
                                " Check the table if you want to update the table," +
                                " or uncheck the column if you don't want to update the column.";
                        target.appendJavaScript(";alert('" + alertString + "');");
                        alert = true;
                        return alert;
                    }
                    
                    if (column.checked == true && schema.checked == false)
                    {
                        alertString = "Can not update column " +
                                column.table_name + "." + column.column_name +
                                " because schema " + schema.schema_name +
                                " is not checked.\n" +
                                " Check the schema if you want to update the schema," +
                                " or uncheck the column if you don't want to update the column.";
                        target.appendJavaScript(";alert('" + alertString + "');");
                        alert = true;
                        return alert;
                    }
                }
            }
        }
        return alert;
    }
    /*
    public void autoUpdateTapSchema(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName)
            throws Exception
    {
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        
        Statement statement2 = null;
        Statement statement3 = null;
        Statement statement4 = null;
        Statement statement5 = null;
        ResultSet resultSet = null;
        ResultSet resultSet2 = null;
        //ResultSet resultSet3 = null;
        ResultSet resultSet4 = null;
        
        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );
            
            //query to write the database name in tap_schema.schemas
            statement = connection.createStatement();
            String queryString = "INSERT INTO " + tapschemaName + ".schemas "
                    + "(schema_name, utype, description, schemaID) VALUES('"
                    + tapschemaName + "', "
                    + "null, "
                    + "null, "
                    + "null);";
            statement.executeUpdate(queryString);
        
            //queries to read the column names of each table in the database
            //and queries to write the table names in tap_schema.tables
            //(for each table write also the schema_name)
            statement = connection.createStatement();
            queryString = "SHOW TABLES FROM " + tapschemaName + ";";
            resultSet = statement.executeQuery(queryString);
            
            while(resultSet.next())
            {
                String queryString2 = "INSERT INTO " + tapschemaName + ".tables " +
                        "(schema_name, table_name, table_type, utype, description, tableID) "
                        + "VALUES('"
                        + tapschemaName + "', '"
                        + tapschemaName + "." + resultSet.getString(1) + "', "
                        + "'table', "
                        + "null, "
                        + "null, "
                        + "null);";
                statement2 = connection.createStatement();
                statement2.executeUpdate(queryString2);
            }
        
            //queries to read the column names of each table in the database
            //and queries to write the column names in tap_schema.columns
            //(for each table write also the full table_name with the schema)
            statement = connection.createStatement();
            queryString = "SHOW TABLES FROM " + tapschemaName + ";";
            resultSet = statement.executeQuery(queryString);
            
            while(resultSet.next())
            {
                String tableName = resultSet.getString(1);
                statement2 = connection.createStatement();
                String queryString2 = "SHOW COLUMNS FROM `" + tableName
                        + "` FROM " + tapschemaName + ";";
                resultSet2 = statement2.executeQuery(queryString2);
                
                statement4 = connection.createStatement();
                String queryString4 = "SHOW KEYS FROM `" + tableName
                        + "` FROM " + tapschemaName + ";";
                resultSet4 = statement4.executeQuery(queryString4);
                ArrayList<String> keysList = new ArrayList<String>();
                while(resultSet4.next())
                {
                    keysList.add(resultSet4.getString("Column_name"));
                }
                
                while(resultSet2.next())
                {
                    int indexed = 0;
                    if (resultSet2.getString("Key").equals("PRI"))
                    {
                        indexed = 1;
                    }
                    for(int i = 0; i < keysList.size(); i++)
                    {
                        if(resultSet2.getString(2).equals(keysList.get(i)))
                        {
                            indexed = 1;
                            break;
                        }
                    }
                    
                    Integer size;
                    String datatype;
                    if(resultSet2.getString(2).startsWith("int")
                            || resultSet2.getString(2).startsWith("INT"))
                    {
                        size = null;
                        datatype = "adql:INTEGER";
                        //datatype = "votable:int";
                        //datatype = "int";
                    }
                    else if(resultSet2.getString(2).startsWith("smallint")
                            || resultSet2.getString(2).startsWith("SMALLINT"))
                    {
                        size = null;
                        datatype = "adql:SMALLINT";
                        //datatype = "votable:short";
                        //datatype = "short";
                    }
                    else if(resultSet2.getString(2).startsWith("bigint")
                            || resultSet2.getString(2).startsWith("BIGINT"))
                    {
                        size = null;
                        datatype = "adql:BIGINT";
                        //datatype = "votable:long";
                        //datatype = "long";
                    }
                    else if(resultSet2.getString(2).startsWith("float")
                            || resultSet2.getString(2).startsWith("FLOAT"))
                    {
                        size = null;
                        datatype = "adql:REAL";
                        //datatype = "votable:float";
                        //datatype = "float";
                    }
                    else if(resultSet2.getString(2).contains("char")
                            || resultSet2.getString(2).contains("CHAR"))
                    {
                        int beginIndex = resultSet2.getString(2).indexOf('(');
                        int endIndex = resultSet2.getString(2).indexOf(')');
                        size = Integer.parseInt(resultSet2.getString(2)
                                .substring(beginIndex + 1, endIndex));
                        datatype = "adql:CHAR";
                        //datatype = "votable:char";
                        //datatype = "char";
                    }
                    else if(resultSet2.getString(2).contains("timestamp")
                            || resultSet2.getString(2).contains("TIMESTAMP"))
                    {
                        size = null;
                        datatype = "adql:TIMESTAMP";
                    }
                    else
                    {
                        size = null;
                        datatype = "adql:" + resultSet2.getString(2).toUpperCase();
                        //datatype = "votable:" + resultSet2.getString(2);
                        //datatype = resultSet2.getString(2);
                    }
                    
                    String queryString3 = "INSERT INTO " + tapschemaName + ".columns "
                        + "(table_name, column_name, utype, ucd, unit, description, "
                        + "datatype, size, principal, indexed, std, columnID) "
                        + "VALUES('"
                        + tapschemaName + "." + tableName + "', '"
                        + resultSet2.getString(1) + "', "
                        + "null, "
                        + "null, "
                        + "null, "
                        + "null, '"
                        + datatype + "', "
                        + size + ", "
                        + 0 + ", "
                        //+ 0 + ", "
                        + indexed + ", "
                        + 1 + ", "
                        + "null);";
                    statement3 = connection.createStatement();
                    statement3.executeUpdate(queryString3);
                }
            }
            
            //queries to read the key names of each table in the database
            //and queries to write the key names in tap_schema.keys
            //and tap_schema.key_columns
            //(for each table write also the full table_name with the schema)
            statement = connection.createStatement();
            queryString = "SELECT k.TABLE_NAME, k.COLUMN_NAME, "
                    + "k.REFERENCED_TABLE_NAME, k.REFERENCED_COLUMN_NAME "
                    + "FROM information_schema.TABLE_CONSTRAINTS i "
                    + "LEFT JOIN information_schema.KEY_COLUMN_USAGE k "
                    + "ON i.CONSTRAINT_NAME = k.CONSTRAINT_NAME "
                    + "WHERE i.CONSTRAINT_TYPE = 'FOREIGN KEY' "
                    + "AND i.TABLE_SCHEMA = '" + tapschemaName + "' "
                    + "AND k.TABLE_SCHEMA = '" + tapschemaName + "';";
            resultSet = statement.executeQuery(queryString);
            
            while(resultSet.next())
            {
                String fromTable = resultSet.getString("k.TABLE_NAME");
                String targetTable = resultSet.getString("k.REFERENCED_TABLE_NAME");
                String fromColumn = resultSet.getString("k.COLUMN_NAME"); 
                String targetColumn = resultSet.getString("k.REFERENCED_COLUMN_NAME");
                
                //calculate the max key_ID
                statement4 = connection.createStatement();
                String queryString4 = "SELECT key_ID FROM " + tapschemaName + ".keys;";
                resultSet4 = statement4.executeQuery(queryString4);
                ArrayList<Integer> keyIDList = new ArrayList<Integer>();
                while(resultSet4.next())
                {
                    keyIDList.add(Integer.parseInt(resultSet4.getString(1)));
                }
                Integer max = 0;
                if(keyIDList.size() > 0)
                {
                    max = keyIDList.get(0);
                }

                if(max != null)
                {
                    for (int m = 1; m < keyIDList.size(); m++)
                    {
                        if(keyIDList.get(m) > max)
                        {
                            max = keyIDList.get(m);
                        }
                    }
                }

                //obtain the new key_ID
                if(max != null)
                {
                    max = max + 1;
                }
                else
                {
                    max = 1;
                }
                
                //write the key into tap_schema.keys
                String queryString3 = "INSERT INTO " + tapschemaName + ".keys " +
                        "(key_id, from_table, target_table, utype, description, keyID) "
                        + "VALUES('"
                        + max.toString() + "', '"
                        + tapschemaName + "." + fromTable + "', '"
                        + tapschemaName + "." + targetTable + "', "
                        + "null, "
                        + "null, "
                        + "null);";
                statement3 = connection.createStatement();
                statement3.executeUpdate(queryString3);
                
                //write the key into tap_schema.key_columns
                String queryString5 = "INSERT INTO " + tapschemaName + ".key_columns " +
                        "(key_id, from_column, target_column, key_columnID) "
                        + "VALUES('"
                        + max.toString() + "', '"
                        + fromColumn + "', '"
                        + targetColumn + "', "
                        + "null);";
                statement5 = connection.createStatement();
                statement5.executeUpdate(queryString5);
            }
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            resultSet4.close();
            //resultSet3.close();
            resultSet2.close();
            resultSet.close();
            statement5.close();
            statement4.close();
            statement3.close();
            statement2.close();
            statement.close();
            connection.close();
        }
    }
    */
    
    /**
     * inserts the tapschema database default metadata into the tapschema database itself.
     * 
     * @param hostname
     * @param port
     * @param username
     * @param password
     * @param tapschemaName
     * @throws Exception 
     */
    public void selfUpdateTapSchema(
            String hostname,
            String port,
            String username,
            String password,
            String tapschemaName)
            throws Exception
    {
        String url = "jdbc:mysql://" + hostname + ":" + port + "/";
        Connection connection = null;
        Statement statement = null;
        
        try
        {
            Class.forName("com.mysql.jdbc.Driver");
            connection = DriverManager.getConnection(
                    url,
                    username,
                    password
            );
            
            // inserts tapschema default metadata into schemas table
            statement = connection.createStatement();
            String queryString = "INSERT INTO " + tapschemaName + ".`schemas` " +
                    "(schema_name, description, utype, schemaID) VALUES " +
                    "('" + tapschemaName + "', 'a special schema to describe a TAP tableset', null, null);";
            statement.executeUpdate(queryString);

            // inserts tapschema default metadata into tables table
            queryString = "INSERT INTO " + tapschemaName + ".tables " +
                    "(schema_name, table_name, table_type, description, utype, tableID) VALUES " +
                    "('" + tapschemaName + "', '" + tapschemaName + ".schemas', 'table', 'description of schemas in this tableset', null, null), " +
                    "('" + tapschemaName + "', '" + tapschemaName + ".tables', 'table', 'description of tables in this tableset', null, null), " +
                    "('" + tapschemaName + "', '" + tapschemaName + ".columns', 'table', 'description of columns in this tableset', null, null), " +
                    "('" + tapschemaName + "', '" + tapschemaName + ".keys', 'table', 'description of foreign keys in this tableset', null, null), " +
                    "('" + tapschemaName + "', '" + tapschemaName + ".key_columns', 'table', 'description of foreign key columns in this tableset', null, null);";
            statement.executeUpdate(queryString);

            // inserts tapschema default metadata into columns table
            queryString = "INSERT INTO " + tapschemaName + ".columns " +
                    "(table_name, column_name, description, utype, ucd, unit, datatype, size, principal, indexed, std, columnID) VALUES " + 
                    "('" + tapschemaName + ".schemas', 'schema_name', 'schema name for reference to TAP_SCHEMA.schemas', null, null, null, 'adql:VARCHAR', 64, 1, 1, 1, null), " + 
                    "('" + tapschemaName + ".schemas', 'utype', 'lists the utypes of schemas in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".schemas', 'description', 'describes schemas in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".schemas', 'schemaID', null, null, null, null, 'adql:BIGINT', 0, 0, 0, 0, null), " +
                    "('" + tapschemaName + ".tables', 'schema_name', 'the schema this table belongs to', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".tables', 'table_name', 'the fully qualified table name', null, null, null, 'adql:VARCHAR', 128, 1, 1, 1, null), " +
                    "('" + tapschemaName + ".tables', 'table_type', 'one of: table view', null, null, null, 'adql:VARCHAR', 8, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".tables', 'utype', 'lists the utype of tables in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".tables', 'description', 'describes tables in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".tables', 'tableID', null, null, null, null, 'adql:BIGINT', 0, 0, 0, 0, null), " +
                    "('" + tapschemaName + ".columns', 'table_name', 'the table this column belongs to', null, null, null, 'adql:VARCHAR', 128, 1, 1, 1, null), " +
                    "('" + tapschemaName + ".columns', 'column_name', 'the column name', null, null, null, 'adql:VARCHAR', 64, 1, 1, 1, null), " +
                    "('" + tapschemaName + ".columns', 'utype', 'lists the utypes of columns in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'ucd', 'lists the UCDs of columns in the tableset', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'unit', 'lists the unit used for column values in the tableset', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'description', 'describes the columns in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'datatype', 'lists the ADQL datatype of columns in the tableset', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'size', 'lists the size of variable-length columns in the tableset', null, null, null, 'adql:INTEGER', null, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'principal', 'a principal column; 1 means 1, 0 means 0', null, null, null, 'adql:INTEGER', null, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'indexed', 'an indexed column; 1 means 1, 0 means 0', null, null, null, 'adql:INTEGER', null, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'std', 'a standard column; 1 means 1, 0 means 0', null, null, null, 'adql:INTEGER', null, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".columns', 'columnID', null, null, null, null, 'adql:BIGINT', 0, 0, 0, 0, null), " +
                    "('" + tapschemaName + ".keys', 'key_id', 'unique key to join to TAP_SCHEMA.key_columns', null, null, null, 'adql:VARCHAR', 64, 1, 1, 1, null), " +
                    "('" + tapschemaName + ".keys', 'from_table', 'the table with the foreign key', null, null, null, 'adql:VARCHAR', 128, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".keys', 'target_table', 'the table with the primary key', null, null, null, 'adql:VARCHAR', 128, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".keys', 'utype', 'lists the utype of keys in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".keys', 'description', 'describes keys in the tableset', null, null, null, 'adql:VARCHAR', 512, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".keys', 'keyID', null, null, null, null, 'adql:BIGINT', 0, 0, 0, 0, null), " +
                    "('" + tapschemaName + ".key_columns', 'key_id', 'key to join to TAP_SCHEMA.keys', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".key_columns', 'from_column', 'column in the from_table', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".key_columns', 'target_column', 'column in the target_table', null, null, null, 'adql:VARCHAR', 64, 0, 0, 1, null), " +
                    "('" + tapschemaName + ".key_columns', 'key_columnID', null, null, null, null, 'adql:BIGINT', 0, 0, 0, 0, null);";
            statement.executeUpdate(queryString);

            // inserts tapschema default metadata into keys table
            queryString = "INSERT INTO " + tapschemaName + ".keys " +
                    "(key_id, from_table, target_table, keyID) VALUES " +
                    "('1', '" + tapschemaName + ".tables', '" + tapschemaName + ".schemas', null), " +
                    "('2', '" + tapschemaName + ".columns', '" + tapschemaName + ".tables', null), " +
                    "('3', '" + tapschemaName + ".keys', '" + tapschemaName + ".tables', null), " +
                    "('4', '" + tapschemaName + ".keys', '" + tapschemaName + ".tables', null), " +
                    "('5', '" + tapschemaName + ".key_columns', '" + tapschemaName + ".keys', null);";
            statement.executeUpdate(queryString);

            // inserts tapschema default metadata into key_columns table
            queryString = "INSERT INTO " + tapschemaName + ".key_columns " +
                    "(key_id, from_column, target_column, key_columnID) VALUES " + 
                    "('1', 'schema_name', 'schema_name', null), " +
                    "('2', 'table_name', 'table_name', null), " +
                    "('3', 'from_table', 'table_name', null), " +
                    "('4', 'target_table', 'table_name', null), " +
                    "('5', 'key_id', 'key_id', null);";
            statement.executeUpdate(queryString);
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
            throw new Exception(exception.getMessage());
        }
        finally
        {
            statement.close();
            connection.close();
        }
    }
                     
}
